@@ -1,6 +1,6 @@
 # kas - setup tool for bitbake based projects
 #
-# Copyright (c) Siemens AG, 2017-2018
+# Copyright (c) Siemens AG, 2017-2020
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -30,6 +30,8 @@ import logging
 import tempfile
 import asyncio
 import errno
+import pathlib
+import signal
 from subprocess import Popen, PIPE
 from .context import get_context
 
@@ -92,6 +94,13 @@ async def run_cmd_async(cmd, cwd, env=None, fail=True, liveupdate=True):
 
     logo = LogOutput(liveupdate)
 
+    try:
+        orig_fd = signal.set_wakeup_fd(-1, warn_on_full_buffer=False)
+        signal.set_wakeup_fd(orig_fd, warn_on_full_buffer=False)
+    except TypeError:
+        # Python < 3.7 - we tried our best
+        pass
+
     try:
         process = await asyncio.create_subprocess_exec(
             *cmd,
@@ -108,10 +117,11 @@ async def run_cmd_async(cmd, cwd, env=None, fail=True, liveupdate=True):
             raise ex
         return (errno.EPERM, str(ex))
 
-    await asyncio.wait([
-        _read_stream(process.stdout, logo.log_stdout),
-        _read_stream(process.stderr, logo.log_stderr)
-    ])
+    tasks = [
+        asyncio.ensure_future(_read_stream(process.stdout, logo.log_stdout)),
+        asyncio.ensure_future(_read_stream(process.stderr, logo.log_stderr))
+    ]
+    await asyncio.wait(tasks)
     ret = await process.wait()
 
     if ret and fail:
@@ -149,23 +159,16 @@ def find_program(paths, name):
     return None
 
 
-def _create_task(routine):
-    try:
-        creation_func = asyncio.ensure_future
-    except AttributeError:
-        # for Python < 3.5, avoiding the keyword 'async' introduced in 3.7
-        creation_func = getattr(asyncio, 'async')
-
-    return creation_func(routine)
-
-
 def repos_fetch(repos):
     """
         Fetches the list of repositories to the kas_work_dir.
     """
+    if len(repos) == 0:
+        return
+
     tasks = []
     for repo in repos:
-        tasks.append(_create_task(repo.fetch_async()))
+        tasks.append(asyncio.ensure_future(repo.fetch_async()))
 
     loop = asyncio.get_event_loop()
     loop.run_until_complete(asyncio.wait(tasks))
@@ -179,9 +182,12 @@ def repos_apply_patches(repos):
     """
         Applies the patches to the repositories.
     """
+    if len(repos) == 0:
+        return
+
     tasks = []
     for repo in repos:
-        tasks.append(_create_task(repo.apply_patches_async()))
+        tasks.append(asyncio.ensure_future(repo.apply_patches_async()))
 
     loop = asyncio.get_event_loop()
     loop.run_until_complete(asyncio.wait(tasks))
@@ -191,7 +197,7 @@ def repos_apply_patches(repos):
             sys.exit(task.result())
 
 
-def get_build_environ():
+def get_build_environ(build_system):
     """
         Creates the build environment variables.
     """
@@ -199,9 +205,15 @@ def get_build_environ():
     # creates the conf directory
 
     init_repo = None
+    if build_system in ['openembedded', 'oe']:
+        scripts = ['oe-init-build-env']
+    elif build_system == 'isar':
+        scripts = ['isar-init-build-env']
+    else:
+        scripts = ['oe-init-build-env', 'isar-init-build-env']
     permutations = \
         [(repo, script) for repo in get_context().config.get_repos()
-         for script in ['oe-init-build-env', 'isar-init-build-env']]
+         for script in scripts]
     for (repo, script) in permutations:
         if os.path.exists(repo.path + '/' + script):
             if init_repo:
@@ -215,23 +227,22 @@ def get_build_environ():
         logging.error('Did not find any init-build-env script')
         sys.exit(1)
 
-    get_bb_env_file = tempfile.mktemp()
-    with open(get_bb_env_file, 'w') as fds:
+    with tempfile.TemporaryDirectory() as temp_dir:
         script = """#!/bin/bash
         set -e
         source %s $1 > /dev/null
         env
         """ % init_script
-        fds.write(script)
-    os.chmod(get_bb_env_file, 0o775)
 
-    env = {}
-    env['PATH'] = '/usr/sbin:/usr/bin:/sbin:/bin'
+        get_bb_env_file = pathlib.Path(temp_dir) / "get_bb_env"
+        get_bb_env_file.write_text(script)
+        get_bb_env_file.chmod(0o775)
 
-    (_, output) = run_cmd([get_bb_env_file, get_context().build_dir],
-                          cwd=init_repo.path, env=env, liveupdate=False)
+        env = {}
+        env['PATH'] = '/usr/sbin:/usr/bin:/sbin:/bin'
 
-    os.remove(get_bb_env_file)
+        (_, output) = run_cmd([str(get_bb_env_file), get_context().build_dir],
+                              cwd=init_repo.path, env=env, liveupdate=False)
 
     env = {}
     for line in output.splitlines():
@@ -263,6 +274,15 @@ def get_build_environ():
     return env
 
 
+def ssh_add_key_file(env, key_path):
+    """
+        Adds an ssh key file to the ssh-agent
+    """
+    with open(key_path) as f:
+        key = f.read()
+        ssh_add_key(env, key)
+
+
 def ssh_add_key(env, key):
     """
         Adds an ssh key to the ssh-agent
@@ -302,20 +322,32 @@ def ssh_setup_agent(envkeys=None):
         Starts the ssh-agent
     """
     env = get_context().environ
-    envkeys = envkeys or ['SSH_PRIVATE_KEY']
+    envkeys = envkeys or ['SSH_PRIVATE_KEY', 'SSH_PRIVATE_KEY_FILE']
     output = os.popen('ssh-agent -s').readlines()
     for line in output:
         matches = re.search(r"(\S+)\=(\S+)\;", line)
         if matches:
             env[matches.group(1)] = matches.group(2)
 
+    found = False
     for envkey in envkeys:
-        key = os.environ.get(envkey)
-        if key:
-            logging.info("adding SSH key")
-            ssh_add_key(env, key)
+        if envkey == 'SSH_PRIVATE_KEY_FILE':
+            key_path = os.environ.get(envkey)
+            if key_path:
+                found = True
+                logging.info("adding SSH key")
+                ssh_add_key_file(env, key_path)
         else:
-            logging.warning('%s is missing', envkey)
+            key = os.environ.get(envkey)
+            if key:
+                found = True
+                logging.info("adding SSH key")
+                ssh_add_key(env, key)
+
+    if found is not True:
+        warning = "None of the following environment keys were set: " + \
+            ", ".join(envkeys)
+        logging.warning(warning)
 
 
 def ssh_no_host_key_check():
@@ -329,10 +361,18 @@ def ssh_no_host_key_check():
         fds.write('Host *\n\tStrictHostKeyChecking no\n\n')
 
 
-def kasplugin(plugin_class):
-    """
-        A decorator that registers kas plugins
-    """
-    if not hasattr(kasplugin, 'plugins'):
-        setattr(kasplugin, 'plugins', [])
-    getattr(kasplugin, 'plugins').append(plugin_class)
+def setup_parser_common_args(parser):
+    parser.add_argument('config',
+                        help='Config file, using .config.yaml if none is '
+                        'specified and using the current directory as '
+                        'repository anchor',
+                        nargs='?')
+    parser.add_argument('--skip',
+                        help='Skip build steps',
+                        default=[])
+    parser.add_argument('--force-checkout', action='store_true',
+                        help='Always checkout the desired refspec of each '
+                        'repository, discarding any local changes')
+    parser.add_argument('--update', action='store_true',
+                        help='Pull new upstream changes to the desired '
+                        'refspec even if it is already checked out locally')
